@@ -1,38 +1,31 @@
+use std::cmp::min;
 use std::fmt::{Display, Error, Formatter};
 use std::sync::Arc;
 
-use fuzzy_matcher::clangd::ClangdMatcher;
-use fuzzy_matcher::skim::SkimMatcherV2;
-use fuzzy_matcher::FuzzyMatcher;
+#[cfg(feature = "nightly-frizbee")]
+use crate::fuzzy_matcher::frizbee::FrizbeeMatcher;
+use crate::fuzzy_matcher::{FuzzyMatcher, IndexType, ScoreType, clangd::ClangdMatcher, skim::SkimMatcherV2};
 
 use crate::item::RankBuilder;
 use crate::{CaseMatching, MatchEngine};
 use crate::{MatchRange, MatchResult, SkimItem};
-use bitflags::_core::cmp::min;
 
 //------------------------------------------------------------------------------
-#[derive(Debug, Copy, Clone)]
+/// Fuzzy matching algorithm to use
+#[derive(Debug, Copy, Clone, Default, PartialEq)]
+#[cfg_attr(feature = "cli", derive(clap::ValueEnum))]
+#[cfg_attr(feature = "cli", clap(rename_all = "snake_case"))]
 pub enum FuzzyAlgorithm {
+    /// Original skim fuzzy matching algorithm (v1)
     SkimV1,
+    /// Improved skim fuzzy matching algorithm (v2, default)
+    #[default]
     SkimV2,
+    /// Clangd fuzzy matching algorithm
     Clangd,
-}
-
-impl FuzzyAlgorithm {
-    pub fn of(algorithm: &str) -> Self {
-        match algorithm.to_ascii_lowercase().as_ref() {
-            "skim_v1" => FuzzyAlgorithm::SkimV1,
-            "skim_v2" | "skim" => FuzzyAlgorithm::SkimV2,
-            "clangd" => FuzzyAlgorithm::Clangd,
-            _ => FuzzyAlgorithm::SkimV2,
-        }
-    }
-}
-
-impl Default for FuzzyAlgorithm {
-    fn default() -> Self {
-        FuzzyAlgorithm::SkimV2
-    }
+    /// Frizbee matching algorithm, typo resistant
+    /// Will fallback to SkimV2 if the feature is not enabled
+    Frizbee,
 }
 
 const BYTES_1M: usize = 1024 * 1024 * 1024;
@@ -70,9 +63,19 @@ impl FuzzyEngineBuilder {
 
     #[allow(deprecated)]
     pub fn build(self) -> FuzzyEngine {
-        use fuzzy_matcher::skim::SkimMatcher;
-        let matcher: Box<dyn FuzzyMatcher> = match self.algorithm {
-            FuzzyAlgorithm::SkimV1 => Box::new(SkimMatcher::default()),
+        use crate::fuzzy_matcher::skim::SkimMatcher;
+        #[allow(unused_mut)]
+        let mut algorithm = self.algorithm;
+        #[cfg(not(feature = "nightly-frizbee"))]
+        if algorithm == FuzzyAlgorithm::Frizbee {
+            warn!("Frizbee algorithm not enabled, using SkimV2");
+            algorithm = FuzzyAlgorithm::SkimV2;
+        }
+        let matcher: Box<dyn FuzzyMatcher> = match algorithm {
+            FuzzyAlgorithm::SkimV1 => {
+                debug!("Initialized SkimV1 algorithm");
+                Box::new(SkimMatcher::default())
+            }
             FuzzyAlgorithm::SkimV2 => {
                 let matcher = SkimMatcherV2::default().element_limit(BYTES_1M);
                 let matcher = match self.case {
@@ -80,6 +83,7 @@ impl FuzzyEngineBuilder {
                     CaseMatching::Ignore => matcher.ignore_case(),
                     CaseMatching::Smart => matcher.smart_case(),
                 };
+                debug!("Initialized SkimV2 algorithm");
                 Box::new(matcher)
             }
             FuzzyAlgorithm::Clangd => {
@@ -89,8 +93,15 @@ impl FuzzyEngineBuilder {
                     CaseMatching::Ignore => matcher.ignore_case(),
                     CaseMatching::Smart => matcher.smart_case(),
                 };
+                debug!("Initialized Clangd algorithm");
                 Box::new(matcher)
             }
+            FuzzyAlgorithm::Frizbee => {
+                #[cfg(not(feature = "nightly-frizbee"))]
+                unreachable!();
+                #[cfg(feature = "nightly-frizbee")]
+                Box::new(FrizbeeMatcher::default())
+            }
         };
 
         FuzzyEngine {
@@ -101,6 +112,7 @@ impl FuzzyEngineBuilder {
     }
 }
 
+/// The fuzzy matching engine
 pub struct FuzzyEngine {
     query: String,
     matcher: Box<dyn FuzzyMatcher>,
@@ -108,11 +120,12 @@ pub struct FuzzyEngine {
 }
 
 impl FuzzyEngine {
+    /// Returns a default builder for chaining
     pub fn builder() -> FuzzyEngineBuilder {
         FuzzyEngineBuilder::default()
     }
 
-    fn fuzzy_match(&self, choice: &str, pattern: &str) -> Option<(i64, Vec<usize>)> {
+    fn fuzzy_match(&self, choice: &str, pattern: &str) -> Option<(ScoreType, Vec<IndexType>)> {
         if pattern.is_empty() {
             return Some((0, Vec::new()));
         } else if choice.is_empty() {
@@ -146,19 +159,23 @@ impl MatchEngine for FuzzyEngine {
             }
         }
 
-        if matched_result == None {
-            return None;
-        }
-
-        let (score, matched_range) = matched_result.unwrap();
+        let (score, matched_range) = matched_result?;
 
+        trace!("matched range {matched_range:?}");
         let begin = *matched_range.first().unwrap_or(&0);
         let end = *matched_range.last().unwrap_or(&0);
 
         let item_len = item_text.len();
+
+        // Use individual character indices for highlighting instead of byte range
+        // This allows each matched character to be highlighted individually
+        let matched_range = MatchRange::Chars(matched_range);
+
         Some(MatchResult {
-            rank: self.rank_builder.build_rank(score as i32, begin, end, item_len),
-            matched_range: MatchRange::Chars(matched_range),
+            rank: self
+                .rank_builder
+                .build_rank(score as i32, begin, end, item_len, item.get_index()),
+            matched_range,
         })
     }
 }
