@@ -1,6 +1,7 @@
 //! Skim item helpers
 //! Including the DefaultSkimItem
 use crate::field::{FieldRange, parse_matching_fields, parse_transform_fields};
+use crate::tui::util::merge_styles;
 use crate::{DisplayContext, SkimItem};
 use ansi_to_tui::IntoText;
 use ratatui::text::{Line, Span};
@@ -230,18 +231,10 @@ impl SkimItem for DefaultSkimItem {
                                 // Flush normal content if any
                                 if !current_content.is_empty() {
                                     // Combine ANSI style with context base_style
-                                    let mut combined_base_style = base_style;
-                                    if let Some(bg) = context.base_style.bg {
-                                        combined_base_style = combined_base_style.bg(bg);
-                                    }
-                                    if let Some(fg) = context.base_style.fg
-                                        && base_style.fg.is_none()
-                                    {
-                                        combined_base_style = combined_base_style.fg(fg);
-                                    }
-                                    combined_base_style =
-                                        combined_base_style.add_modifier(context.base_style.add_modifier);
-                                    new_spans.push(Span::styled(current_content.clone(), combined_base_style));
+                                    new_spans.push(Span::styled(
+                                        current_content.clone(),
+                                        merge_styles(context.base_style, base_style),
+                                    ));
                                     current_content.clear();
                                 }
                                 highlighted_content.push(ch);
@@ -249,17 +242,10 @@ impl SkimItem for DefaultSkimItem {
                                 // Flush highlighted content if any
                                 if !highlighted_content.is_empty() {
                                     // Combine styles: use highlight bg, preserve ANSI fg and modifiers
-                                    let mut combined_style = base_style;
-                                    if let Some(bg) = context.style.bg {
-                                        combined_style = combined_style.bg(bg);
-                                    }
-                                    if let Some(fg) = context.style.fg
-                                        && base_style.fg.is_none()
-                                    {
-                                        combined_style = combined_style.fg(fg);
-                                    }
-                                    combined_style = combined_style.add_modifier(context.style.add_modifier);
-                                    new_spans.push(Span::styled(highlighted_content.clone(), combined_style));
+                                    new_spans.push(Span::styled(
+                                        highlighted_content.clone(),
+                                        merge_styles(base_style, context.matched_syle),
+                                    ));
                                     highlighted_content.clear();
                                 }
                                 current_content.push(ch);
@@ -270,31 +256,17 @@ impl SkimItem for DefaultSkimItem {
                         // Flush remaining content
                         if !current_content.is_empty() {
                             // Combine ANSI style with context base_style
-                            let mut combined_base_style = base_style;
-                            if let Some(bg) = context.base_style.bg {
-                                combined_base_style = combined_base_style.bg(bg);
-                            }
-                            if let Some(fg) = context.base_style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_base_style = combined_base_style.fg(fg);
-                            }
-                            combined_base_style = combined_base_style.add_modifier(context.base_style.add_modifier);
-                            new_spans.push(Span::styled(current_content, combined_base_style));
+                            new_spans.push(Span::styled(
+                                current_content,
+                                merge_styles(context.base_style, base_style),
+                            ));
                         }
                         if !highlighted_content.is_empty() {
                             // Combine styles: use highlight bg, preserve ANSI fg and modifiers
-                            let mut combined_style = base_style;
-                            if let Some(bg) = context.style.bg {
-                                combined_style = combined_style.bg(bg);
-                            }
-                            if let Some(fg) = context.style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_style = combined_style.fg(fg);
-                            }
-                            combined_style = combined_style.add_modifier(context.style.add_modifier);
-                            new_spans.push(Span::styled(highlighted_content, combined_style));
+                            new_spans.push(Span::styled(
+                                highlighted_content,
+                                merge_styles(base_style, context.matched_syle),
+                            ));
                         }
                     }
 
@@ -327,45 +299,18 @@ impl SkimItem for DefaultSkimItem {
 
                         if !before.is_empty() {
                             // Combine ANSI style with context base_style
-                            let mut combined_base_style = base_style;
-                            if let Some(bg) = context.base_style.bg {
-                                combined_base_style = combined_base_style.bg(bg);
-                            }
-                            if let Some(fg) = context.base_style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_base_style = combined_base_style.fg(fg);
-                            }
-                            combined_base_style = combined_base_style.add_modifier(context.base_style.add_modifier);
-                            new_spans.push(Span::styled(before, combined_base_style));
+                            new_spans.push(Span::styled(before, merge_styles(context.base_style, base_style)));
                         }
                         if !highlighted.is_empty() {
-                            // Combine styles: use highlight bg, preserve ANSI fg and modifiers
-                            let mut combined_style = base_style;
-                            if let Some(bg) = context.style.bg {
-                                combined_style = combined_style.bg(bg);
-                            }
-                            if let Some(fg) = context.style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_style = combined_style.fg(fg);
-                            }
-                            combined_style = combined_style.add_modifier(context.style.add_modifier);
-                            new_spans.push(Span::styled(highlighted, combined_style));
+                            // Combine ANSI style with context matched_syle
+                            new_spans.push(Span::styled(
+                                highlighted,
+                                merge_styles(base_style, context.matched_syle),
+                            ));
                         }
                         if !after.is_empty() {
                             // Combine ANSI style with context base_style
-                            let mut combined_base_style = base_style;
-                            if let Some(bg) = context.base_style.bg {
-                                combined_base_style = combined_base_style.bg(bg);
-                            }
-                            if let Some(fg) = context.base_style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_base_style = combined_base_style.fg(fg);
-                            }
-                            combined_base_style = combined_base_style.add_modifier(context.base_style.add_modifier);
-                            new_spans.push(Span::styled(after, combined_base_style));
+                            new_spans.push(Span::styled(after, merge_styles(context.base_style, base_style)));
                         }
                     }
 
@@ -403,70 +348,24 @@ impl SkimItem for DefaultSkimItem {
 
                         if !before.is_empty() {
                             // Combine ANSI style with context base_style
-                            let mut combined_base_style = base_style;
-                            if let Some(bg) = context.base_style.bg {
-                                combined_base_style = combined_base_style.bg(bg);
-                            }
-                            if let Some(fg) = context.base_style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_base_style = combined_base_style.fg(fg);
-                            }
-                            combined_base_style = combined_base_style.add_modifier(context.base_style.add_modifier);
-                            new_spans.push(Span::styled(before, combined_base_style));
+                            new_spans.push(Span::styled(before, merge_styles(context.base_style, base_style)));
                         }
                         if !highlighted.is_empty() {
-                            // Combine styles: use highlight bg, preserve ANSI fg and modifiers
-                            let mut combined_style = base_style;
-                            if let Some(bg) = context.style.bg {
-                                combined_style = combined_style.bg(bg);
-                            }
-                            if let Some(fg) = context.style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_style = combined_style.fg(fg);
-                            }
-                            combined_style = combined_style.add_modifier(context.style.add_modifier);
-                            new_spans.push(Span::styled(highlighted, combined_style));
+                            // Combine ANSI style with context matched_syle
+                            new_spans.push(Span::styled(
+                                highlighted,
+                                merge_styles(base_style, context.matched_syle),
+                            ));
                         }
                         if !after.is_empty() {
                             // Combine ANSI style with context base_style
-                            let mut combined_base_style = base_style;
-                            if let Some(bg) = context.base_style.bg {
-                                combined_base_style = combined_base_style.bg(bg);
-                            }
-                            if let Some(fg) = context.base_style.fg
-                                && base_style.fg.is_none()
-                            {
-                                combined_base_style = combined_base_style.fg(fg);
-                            }
-                            combined_base_style = combined_base_style.add_modifier(context.base_style.add_modifier);
-                            new_spans.push(Span::styled(after, combined_base_style));
+                            new_spans.push(Span::styled(after, merge_styles(context.base_style, base_style)));
                         }
                     }
 
                     Line::from(new_spans)
                 }
-                crate::Matches::None => {
-                    // No highlighting needed, but apply base_style to all spans
-                    let styled_spans: Vec<Span> = all_spans
-                        .into_iter()
-                        .map(|span| {
-                            let mut combined_style = span.style;
-                            if let Some(bg) = context.base_style.bg {
-                                combined_style = combined_style.bg(bg);
-                            }
-                            if let Some(fg) = context.base_style.fg
-                                && span.style.fg.is_none()
-                            {
-                                combined_style = combined_style.fg(fg);
-                            }
-                            combined_style = combined_style.add_modifier(context.base_style.add_modifier);
-                            Span::styled(span.content, combined_style)
-                        })
-                        .collect();
-                    Line::from(styled_spans)
-                }
+                crate::Matches::None => Line::from(all_spans),
             }
         } else {
             // No ANSI mapping needed, use text as-is
@@ -706,7 +605,7 @@ mod test {
             matches: Matches::CharRange(6, 10),
             container_width: 80,
             base_style: Style::default(),
-            style: Style::default().fg(Color::Yellow),
+            matched_syle: Style::default().fg(Color::Yellow),
         };
 
         // display() should map the match positions back to the original ANSI text
@@ -745,7 +644,7 @@ mod test {
             matches: Matches::CharIndices(vec![1, 2]),
             container_width: 80,
             base_style: Style::default(),
-            style: Style::default().fg(Color::Yellow),
+            matched_syle: Style::default().fg(Color::Yellow),
         };
 
         // display() should map these to positions 6,7 in original text
@@ -815,7 +714,7 @@ mod test {
             matches: Matches::CharIndices(vec![0]),
             container_width: 80,
             base_style: Style::default(),
-            style: Style::default().bg(Color::Yellow),
+            matched_syle: Style::default().bg(Color::Yellow),
         };
 
         let line = item.display(context);
@@ -854,7 +753,7 @@ mod test {
             matches: Matches::CharRange(1, 3),
             container_width: 80,
             base_style: Style::default(),
-            style: Style::default().bg(Color::Yellow),
+            matched_syle: Style::default().bg(Color::Yellow),
         };
 
         let line = item.display(context);
@@ -895,7 +794,7 @@ mod test {
             matches: Matches::ByteRange(1, 3),
             container_width: 80,
             base_style: Style::default(),
-            style: Style::default().bg(Color::Yellow),
+            matched_syle: Style::default().bg(Color::Yellow),
         };
 
         let line = item.display(context);
