@@ -45,6 +45,7 @@ use ratatui::style::Style;
 use ratatui::text::{Line, Span};
 use reader::Reader;
 use tokio::select;
+use tokio::sync::mpsc::error::TryRecvError;
 use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender, unbounded_channel};
 use tui::App;
 use tui::Event;
@@ -58,8 +59,6 @@ pub use crate::skim_item::SkimItem;
 use crate::tui::event::Action;
 
 pub mod binds;
-#[cfg(feature = "cli")]
-pub mod completions;
 mod engine;
 pub mod field;
 pub mod fuzzy_matcher;
@@ -77,6 +76,11 @@ pub mod tmux;
 pub mod tui;
 mod util;
 
+#[cfg(feature = "cli")]
+pub mod completions;
+#[cfg(feature = "cli")]
+pub mod manpage;
+
 //------------------------------------------------------------------------------
 /// Trait for downcasting to concrete types from trait objects
 pub trait AsAny {
@@ -98,7 +102,7 @@ impl<T: Any> AsAny for T {
 
 //------------------------------------------------------------------------------
 // Display Context
-#[derive(Default, Debug)]
+#[derive(Default, Debug, Clone)]
 /// Represents how a query matches an item
 pub enum Matches {
     /// No matches
@@ -112,7 +116,7 @@ pub enum Matches {
     ByteRange(usize, usize),
 }
 
-#[derive(Default)]
+#[derive(Default, Clone)]
 /// Context information for displaying an item
 pub struct DisplayContext {
     /// The match score for this item
@@ -124,7 +128,7 @@ pub struct DisplayContext {
     /// The base style to apply to non-matched portions
     pub base_style: Style,
     /// The style to apply to matched portions
-    pub style: Style,
+    pub matched_syle: Style,
 }
 
 impl DisplayContext {
@@ -144,7 +148,7 @@ impl DisplayContext {
                     res.push_span(Span::styled(span_content.collect::<String>(), self.base_style));
                     let highlighted_char = chars.next().unwrap_or_default().to_string();
 
-                    res.push_span(Span::styled(highlighted_char, self.style));
+                    res.push_span(Span::styled(highlighted_char, self.base_style.patch(self.matched_syle)));
                     prev_index = index + 1;
                 }
                 res.push_span(Span::styled(chars.collect::<String>(), self.base_style));
@@ -161,7 +165,7 @@ impl DisplayContext {
                 ));
                 let highlighted_text = chars.by_ref().take(*end - *start).collect::<String>();
 
-                res.push_span(Span::styled(highlighted_text, self.style));
+                res.push_span(Span::styled(highlighted_text, self.base_style.patch(self.matched_syle)));
                 res.push_span(Span::styled(chars.collect::<String>(), self.base_style));
                 res
             }
@@ -175,7 +179,7 @@ impl DisplayContext {
                 let highlighted_bytes = bytes.by_ref().take(*end - *start).collect();
                 let highlighted_text = String::from_utf8(highlighted_bytes).unwrap();
 
-                res.push_span(Span::styled(highlighted_text, self.style));
+                res.push_span(Span::styled(highlighted_text, self.base_style.patch(self.matched_syle)));
                 res.push_span(Span::styled(
                     String::from_utf8(bytes.collect()).unwrap(),
                     self.base_style,
@@ -348,8 +352,6 @@ impl Skim {
     /// Panics if the tui fails to initilize
     pub fn run_with(options: SkimOptions, source: Option<SkimItemReceiver>) -> Result<SkimOutput> {
         let height = Size::try_from(options.height.as_str())?;
-        let backend = CrosstermBackend::new(std::io::stderr());
-        let mut tui = tui::Tui::new_with_height(backend, height)?;
 
         // application state
         // Initialize theme from options
@@ -369,8 +371,6 @@ impl Skim {
         let mut final_event: Event = Event::Quit;
         let mut final_key: KeyEvent = KeyEvent::new(KeyCode::Enter, KeyModifiers::empty());
         rt.block_on(async {
-            tui.enter()?;
-
             //------------------------------------------------------------------------------
             // reader
             // In interactive mode, expand all placeholders ({}, {q}, etc) with initial query (empty or from --query)
@@ -395,12 +395,21 @@ impl Skim {
             let mut matcher_interval = tokio::time::interval(Duration::from_millis(100));
             let reader_done = Arc::new(AtomicBool::new(false));
             let reader_done_clone = reader_done.clone();
+            let (reader_interrupt_tx, mut reader_interrupt_rx) = unbounded_channel();
 
             let item_pool = app.item_pool.clone();
             tokio::spawn(async move {
                 const BATCH: usize = 4096; // Smaller batches for more responsive updates
                 loop {
+                    if reader_interrupt_rx
+                        .try_recv()
+                        .unwrap_or_else(|e| e == TryRecvError::Disconnected)
+                    {
+                        debug!("stopping reader receiver thread");
+                        break;
+                    }
                     let mut buf = Vec::with_capacity(BATCH);
+                    trace!("getting items");
                     if item_rx.recv_many(&mut buf, BATCH).await > 0 {
                         item_pool.append(buf);
                         trace!("Got new items, len {}", item_pool.len());
@@ -413,6 +422,15 @@ impl Skim {
             // Start matcher initially
             app.restart_matcher(true);
 
+            // Deal with read-0 / select-1
+            let min_items_before_enter = if app.options.exit_0 {
+                1
+            } else if app.options.select_1 {
+                2
+            } else {
+                0
+            };
+
             let listener = if let Some(socket_name) = &listen_socket {
                 debug!("starting listener on socket at {socket_name}");
                 Some(
@@ -431,8 +449,39 @@ impl Skim {
             } else {
                 None
             };
-            let event_tx_clone = tui.event_tx.clone();
 
+            let mut should_enter = true;
+            if min_items_before_enter > 0 {
+                while app.matcher_control.get_num_matched() < min_items_before_enter && app.item_pool.num_not_taken() > 0 && !app.matcher_control.stopped() {
+                    let curr = app.matcher_control.get_num_matched();
+                    tokio::time::sleep(Duration::from_millis(10)).await;
+                    trace!(
+                        "waiting for matcher, stopped: {}, processed: {}, pool: {}, matched: {}, query: {}, reader_done: {}, reader_control_done: {}",
+                        app.matcher_control.stopped(),
+                        app.matcher_control.get_num_processed(),
+                        app.item_pool.num_not_taken(),
+                        curr,
+                        app.input.value,
+                        reader_done.load(std::sync::atomic::Ordering::Relaxed),
+                        reader_control.is_done()
+                    );
+                    app.restart_matcher(false);
+                };
+                trace!("checking for matched item count before entering: {}/{min_items_before_enter}", app.matcher_control.get_num_matched());
+                if app.matcher_control.stopped() && app.matcher_control.get_num_matched() == min_items_before_enter - 1
+                {
+                    app.item_list.items = app.item_list.processed_items.lock().take().unwrap_or_default().items;
+                    debug!("early exit, result: {:?}", app.results());
+                    should_enter = false;
+                    final_event = Event::Action(Action::Accept(None));
+                }
+            }
+
+            if should_enter {
+            let backend = CrosstermBackend::new(std::io::BufWriter::new(std::io::stderr()));
+            let mut tui = tui::Tui::new_with_height(backend, height)?;
+            let event_tx_clone = tui.event_tx.clone();
+            tui.enter()?;
             loop {
                 select! {
                     event = tui.next() => {
@@ -453,7 +502,6 @@ impl Skim {
                             app.restart_matcher(true);
                             // Start a new reader with the new command (no source, using cmd)
                             reader_control = reader.run(item_tx.clone(), new_cmd);
-                            app.status.reading = true;
                             reader_done.store(false, std::sync::atomic::Ordering::Relaxed);
                         }
                         if let Event::Key(k) = &evt {
@@ -468,7 +516,6 @@ impl Skim {
                         } else if ! reader_done.load(std::sync::atomic::Ordering::Relaxed) {
                             reader_done.store(true, std::sync::atomic::Ordering::Relaxed);
                             app.restart_matcher(true);
-                            app.status.reading = false;
                         }
                         app.handle_event(&mut tui, &evt)?;
                     }
@@ -502,7 +549,9 @@ impl Skim {
                     break;
                 }
             }
+            }
             reader_control.kill();
+            let _ = reader_interrupt_tx.send(true);
             eyre::Ok(())
         })?;
 
