@@ -2,24 +2,27 @@ use std::{rc::Rc, sync::Arc};
 
 use indexmap::IndexSet;
 use ratatui::text::{Line, Span};
-use ratatui::widgets::{Clear, List, ListDirection, ListItem, ListState, StatefulWidget, Widget};
+use ratatui::widgets::{Block, Borders, Clear, List, ListDirection, ListItem, ListState, StatefulWidget, Widget};
 use regex::Regex;
 use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender, unbounded_channel};
-use unicode_width::{UnicodeWidthChar, UnicodeWidthStr};
+use unicode_display_width::width as display_width;
 
+use crate::tui::util::char_display_width;
 use crate::{
     DisplayContext, MatchRange, Selector, SkimItem, SkimOptions,
     item::MatchedItem,
     spinlock::SpinLock,
     theme::ColorTheme,
+    tui::BorderType,
     tui::options::TuiLayout,
     tui::util::wrap_text,
     tui::widget::{SkimRender, SkimWidget},
 };
 
 /// Processed items ready for rendering
-struct ProcessedItems {
-    items: Vec<MatchedItem>,
+#[derive(Default)]
+pub(crate) struct ProcessedItems {
+    pub(crate) items: Vec<MatchedItem>,
 }
 
 /// Widget for displaying and managing the list of filtered items
@@ -27,7 +30,7 @@ pub struct ItemList {
     pub(crate) items: Vec<MatchedItem>,
     pub(crate) selection: IndexSet<MatchedItem>,
     pub(crate) tx: UnboundedSender<Vec<MatchedItem>>,
-    processed_items: Arc<SpinLock<Option<ProcessedItems>>>,
+    pub(crate) processed_items: Arc<SpinLock<Option<ProcessedItems>>>,
     pub(crate) direction: ListDirection,
     pub(crate) offset: usize,
     pub(crate) current: usize,
@@ -49,6 +52,8 @@ pub struct ItemList {
     multi_select_icon: String,
     cycle: bool,
     wrap: bool,
+    /// Border type, if borders are enabled
+    pub border: Option<BorderType>,
 }
 
 impl Default for ItemList {
@@ -82,6 +87,7 @@ impl Default for ItemList {
             multi_select_icon: String::from(">"),
             cycle: false,
             wrap: false,
+            border: None,
         }
     }
 }
@@ -92,12 +98,15 @@ impl ItemList {
     fn process_items_task(
         mut rx: UnboundedReceiver<Vec<MatchedItem>>,
         processed_items: Arc<SpinLock<Option<ProcessedItems>>>,
+        no_sort: bool,
     ) {
         while let Some(mut items) = rx.blocking_recv() {
             debug!("Background task: Got {} items to process", items.len());
 
             // Sort items immediately - use stable sort to preserve order for equal ranks
-            items.sort_by_key(|item| item.rank);
+            if !no_sort {
+                items.sort_by_key(|item| item.rank);
+            }
 
             // Write processed items to shared state for render thread
             // Move items instead of cloning for efficiency
@@ -136,7 +145,7 @@ impl ItemList {
         if let Some(ref regex) = self.skip_to_pattern
             && let Some(mat) = regex.find(text)
         {
-            return text[..mat.start()].width_cjk();
+            return display_width(&text[..mat.start()]).try_into().unwrap();
         }
         0
     }
@@ -155,7 +164,7 @@ impl ItemList {
             if ch == '\t' {
                 acc + self.tabstop - (acc % self.tabstop)
             } else {
-                acc + ch.to_string().width_cjk()
+                acc + char_display_width(ch)
             }
         });
 
@@ -205,7 +214,7 @@ impl ItemList {
                 if ch == '\t' {
                     current_width += self.tabstop - (current_width % self.tabstop);
                 } else {
-                    current_width += ch.to_string().width_cjk();
+                    current_width += char_display_width(ch);
                 }
             }
 
@@ -311,7 +320,7 @@ impl ItemList {
             }
 
             current_char_index += span_chars.len();
-            current_width += span_text.width_cjk();
+            current_width += usize::try_from(display_width(span_text)).unwrap();
         }
 
         // Add right indicator if needed
@@ -331,7 +340,7 @@ impl ItemList {
                 let ch_width = if ch == '\t' {
                     self.tabstop - (current_width % self.tabstop)
                 } else {
-                    ch.width_cjk().unwrap_or_default()
+                    char_display_width(ch)
                 };
 
                 if current_width >= target_width {
@@ -357,7 +366,7 @@ impl ItemList {
                 current_width += tab_width;
             } else {
                 result.push(ch);
-                current_width += ch.to_string().width_cjk();
+                current_width += char_display_width(ch)
             }
         }
 
@@ -522,19 +531,20 @@ impl SkimWidget for ItemList {
 
         // Spawn background processing thread with the appropriate configuration
         let processed_items_clone = processed_items.clone();
+        let no_sort = options.no_sort;
         std::thread::spawn(move || {
-            Self::process_items_task(rx, processed_items_clone);
+            Self::process_items_task(rx, processed_items_clone, no_sort);
         });
 
         Self {
             tx,
             processed_items,
-            reserved: options.header_lines,
+            reserved: 0, // header_lines are now displayed in the Header widget, not ItemList
             direction: match options.layout {
                 TuiLayout::Default => ratatui::widgets::ListDirection::BottomToTop,
                 TuiLayout::Reverse | TuiLayout::ReverseList => ratatui::widgets::ListDirection::TopToBottom,
             },
-            current: options.header_lines,
+            current: 0,
             theme,
             multi_select,
             no_hscroll: options.no_hscroll,
@@ -555,16 +565,30 @@ impl SkimWidget for ItemList {
             multi_select_icon: options.multi_select_icon.clone(),
             cycle: options.cycle,
             wrap: options.wrap_items,
+            border: options.border,
         }
     }
 
     fn render(&mut self, area: ratatui::prelude::Rect, buf: &mut ratatui::prelude::Buffer) -> SkimRender {
         let this = &mut *self;
-        this.height = area.height;
+
+        // Calculate inner area if borders are enabled
+        let inner_area = if this.border.is_some() {
+            ratatui::layout::Rect {
+                x: area.x + 1,
+                y: area.y + 1,
+                width: area.width.saturating_sub(2),
+                height: area.height.saturating_sub(2),
+            }
+        } else {
+            area
+        };
+
+        this.height = inner_area.height;
         if this.current < this.offset {
             this.offset = this.current;
-        } else if this.offset + area.height as usize <= this.current {
-            this.offset = this.current - area.height as usize + 1;
+        } else if this.offset + inner_area.height as usize <= this.current {
+            this.offset = this.current - inner_area.height as usize + 1;
         }
 
         // Check for pre-processed items from background thread (non-blocking)
@@ -617,6 +641,15 @@ impl SkimWidget for ItemList {
         };
 
         if this.items.is_empty() {
+            // Still render border if enabled, even with no items
+            if let Some(border_type) = this.border {
+                let block = Block::default()
+                    .borders(Borders::ALL)
+                    .border_type(border_type.into())
+                    .border_style(this.theme.border);
+                Widget::render(Clear, area, buf);
+                Widget::render(block, area, buf);
+            }
             return SkimRender { items_updated };
         }
 
@@ -630,13 +663,13 @@ impl SkimWidget for ItemList {
                 .iter()
                 .enumerate()
                 .skip(this.offset)
-                .take(area.height as usize)
+                .take(inner_area.height as usize)
                 .map(|(idx, item)| {
                     let is_current = idx == this.current;
                     let is_selected = this.selection.contains(item);
 
                     // Reserve 2 characters for cursor indicators ("> " or " >")
-                    let container_width = (area.width as usize)
+                    let container_width = (inner_area.width as usize)
                         .saturating_sub(selector_icon.chars().count() + multi_select_icon.chars().count());
 
                     // Get item text for hscroll calculation
@@ -676,7 +709,7 @@ impl SkimWidget for ItemList {
                         matches,
                         container_width,
                         base_style: if is_current { theme.current } else { theme.normal },
-                        style: if is_current { theme.current_match } else { theme.matched },
+                        matched_syle: if is_current { theme.current_match } else { theme.matched },
                     });
 
                     if !wrap {
@@ -706,7 +739,7 @@ impl SkimWidget for ItemList {
                     spans.extend(display_line.spans);
 
                     if *wrap {
-                        wrap_text(ratatui::text::Text::from(Line::from(spans)), area.width.into()).into()
+                        wrap_text(ratatui::text::Text::from(Line::from(spans)), inner_area.width.into()).into()
                     } else {
                         Line::from(spans).into()
                     }
@@ -717,9 +750,19 @@ impl SkimWidget for ItemList {
         .style(this.theme.normal);
 
         Widget::render(Clear, area, buf);
+
+        // Render border if enabled
+        if let Some(border_type) = this.border {
+            let block = Block::default()
+                .borders(Borders::ALL)
+                .border_type(border_type.into())
+                .border_style(this.theme.border);
+            Widget::render(block, area, buf);
+        }
+
         StatefulWidget::render(
             list,
-            area,
+            inner_area,
             buf,
             &mut ListState::default().with_selected(Some(this.current.saturating_sub(this.offset))),
         );
