@@ -60,7 +60,6 @@ impl<'a> TestHarness<'a> {
             // Run the command and add items
             self.run_command_internal(new_cmd)?;
             self.app.restart_matcher(true);
-            self.app.status.reading = true;
         }
 
         // Let the app handle the event (this may queue more events)
@@ -157,7 +156,9 @@ impl<'a> TestHarness<'a> {
     #[doc(hidden)]
     pub fn snap(&mut self) -> Result<()> {
         self.prepare_snap()?;
-        insta::assert_snapshot!(self.buffer_view());
+        let buf = self.buffer_view();
+        let cursor_pos = format!("cursor: {}x{}", self.app.cursor_pos.0, self.app.cursor_pos.1);
+        insta::assert_snapshot!(buf + &cursor_pos);
         Ok(())
     }
 
@@ -433,7 +434,13 @@ pub fn parse_options(args: &[&str]) -> SkimOptions {
 macro_rules! snap {
     ($harness:ident) => {
         $harness.prepare_snap()?;
-        insta::assert_snapshot!($harness.buffer_view());
+        let buf = $harness.buffer_view();
+        let cursor_pos = format!(
+            "cursor: ({}, {})",
+            $harness.app.cursor_pos.1 + 1,
+            $harness.app.cursor_pos.0 + 1
+        );
+        insta::assert_snapshot!(buf + &cursor_pos);
     };
 }
 
